﻿using Celeste;
using Celeste.Mod;
using FMOD.Studio;
using Monocle;
using MonoMod.Utils;
using System.Collections.Generic;
using System.Reflection;

namespace VanillaTweaks
{
    class Jukebox
    {
        // index, handle
        static Dictionary<int, string> byHandle = new Dictionary<int, string>();

        public static void Load()
        {
            On.Celeste.Audio.SetMusic += Audio_SetMusic;
            On.Celeste.Audio.SetMusicParam += Audio_SetMusicParam;
            Everest.Events.Level.OnCreatePauseMenuButtons += Level_OnCreatePauseMenuButtons;
            On.Celeste.AudioState.Apply += AudioState_Apply;
        }

        public static void Unload()
        {
            On.Celeste.Audio.SetMusic -= Audio_SetMusic;
            On.Celeste.Audio.SetMusicParam -= Audio_SetMusicParam;
            Everest.Events.Level.OnCreatePauseMenuButtons -= Level_OnCreatePauseMenuButtons;
            On.Celeste.AudioState.Apply -= AudioState_Apply;
        }

        private static void AudioState_Apply(On.Celeste.AudioState.orig_Apply orig, AudioState self)
        {
            if (!Jukebox.playing)
            {
                orig(self);
            }
        }

        private static void Audio_SetMusicParam(On.Celeste.Audio.orig_SetMusicParam orig, string path, float value)
        {
            if (Jukebox.playing)
            {
                if (path == "progress")
                    Jukebox.mutedSongProgress = (int)value;
            }
            else
            {
                orig(path, value);
            }
        }

        private static void Level_OnCreatePauseMenuButtons(Level level, TextMenu menu, bool minimal)
        {
            if (VanillaTweaksModule.Settings.ShowJukebox)
            {
                TextMenu.Item item = null;
                menu.Add(item = new TextMenu.Button(Dialog.Clean("VanillaTweaks_Jukebox")).Pressed(delegate
                {
                    menu.RemoveSelf();
                    level.PauseMainMenuOpen = false;
                    Jukebox.JukeboxScreen(level, menu.IndexOf(item), minimal);
                }));
            }

        }

        private static bool Audio_SetMusic(On.Celeste.Audio.orig_SetMusic orig, string path, bool startPlaying, bool allowFadeOut)
        {
            if (!Jukebox.playing)
            {
                return orig(path, startPlaying, allowFadeOut);
            }
            else
            {
                Jukebox.mutedSong = path;
            }
            return false;
        }


        public static void GetByHandle()
        {
            // Get all names
            
            FieldInfo[] fields = typeof(SFX).GetFields(BindingFlags.Static | BindingFlags.Public);
            int i = 0;
            foreach (FieldInfo fieldInfo in fields)
            {
                bool flag = fieldInfo.FieldType == typeof(string);
                if (flag)
                {
                    string text = fieldInfo.GetValue(null).ToString();
                    if ((text.StartsWith("event:/music") || text.StartsWith("event:/new_content/music")) && !text.Contains("cassette") && !text.Contains("dreamblock") && text != "event:/music/lvl3/clean_extended" && text != "event:/music/lvl5/mirror_cutscene")
                    {
                        byHandle.Add(i, fieldInfo.Name);
                        i++;
                    }
                    
                }
            }
        }

        public static void PlayMusic(string song)
        {
            Logger.Log(VanillaTweaksModule.ModName, $"Playing Song: {song}");
            if (!playing)
                mutedSong = Audio.CurrentMusic;
            playing = false;
            Audio.SetMusic(SFX.EventnameByHandle(song), true, true);
            playing = true;
            playingSong = SFX.EventnameByHandle(song);
        }

        private static void StopMusic()
        {
            playing = false;
            //Audio.SetMusic(SFX.EventnameByHandle(Jukebox.prevSong), true, true);
            //Audio.SetMusic(mutedSong, true, true);
            //Audio.SetMusicParam("progress", mutedSongProgress);
            (Engine.Scene as Level).Session.Audio.Music.Progress = mutedSongProgress;
            (Engine.Scene as Level).Session.Audio.Music.Event = SFX.EventnameByHandle(mutedSong);
            (Engine.Scene as Level).Session.Audio.Apply(false);
        }

        private static void SetMusicProgress(int progress)
        {
            playing = false;
            (Engine.Scene as Level).Session.Audio.Music.Progress = progress;
            (Engine.Scene as Level).Session.Audio.Music.Event = SFX.EventnameByHandle(playingSong);
            (Engine.Scene as Level).Session.Audio.Apply(false);
            playing = true;
        }

        private static int GetMusicProgress()
        {
            /*
            float finalvalue;
            float value;
            Audio.CurrentMusicEventInstance.getParameterValue("progress", out value, out finalvalue);
            return (int)value; */
            return (Engine.Scene as Level).Session.Audio.Music.Progress;
        }

        public static bool playing;
        public static string mutedSong;
        public static string playingSong = "NONE";
        public static int mutedSongProgress;

        public static int GetMusicIndex(string music)
        {
            int index = -1;
            //foreach(string song in byHandle.Values)
            if (music == "NONE")
            {
                return 0;
            }
            for(int i = 0; i < byHandle.Count; i++)
            {
                index++;
                //Logger.Log(VanillaTweaksModule.ModName, $"{byHandle[i]}, {music}");
                if (SFX.EventnameByHandle(byHandle[i]) == music)
                {
                    break;
                }
            }

            return index;
        }

        public static string GetMusicAtIndex(int index)
        {
            return byHandle[index];
        }

        public static void JukeboxScreen(Level level, int returnIndex, bool minimal)
        {
            TextMenu menu = new TextMenu();
            // screw the devs for making everything private D:
            DynData<Level> data = new DynData<Level>(level);
            EventInstance PauseSnapshot = data.Get<EventInstance>("PauseSnapshot");
            menu.Add(new TextMenu.Header(Dialog.Clean("VanillaTweaks_Jukebox")));
            menu.Add(new TextMenu.SubHeader(Dialog.Clean("VanillaTweaks_Jukebox_WhatSong")));

            menu.Add(new TextMenu.Slider(Dialog.Clean("VanillaTweaks_Jukebox_Song", null), delegate (int i)
            {
                return GetMusicAtIndex(i);
            }, 0, byHandle.Count - 1, GetMusicIndex(playingSong)).Change(delegate (int i)
            {
                PlayMusic(GetMusicAtIndex(i));
            }).Enter(new System.Action(EnterSound)).Leave(new System.Action(LeaveSound)));

            menu.Add(new TextMenu.Slider(Dialog.Clean("VanillaTweaks_Jukebox_Progress", null), delegate (int i)
            {
                return i.ToString();
            }, 1, 10, GetMusicProgress()).Change(delegate (int i)
            {
                SetMusicProgress(i);
            }));

            menu.Add(new TextMenu.Button(Dialog.Clean("VanillaTweaks_Jukebox_ShutOff")).Pressed(delegate
            {
                StopMusic();
            }));

            menu.OnESC = (menu.OnCancel = delegate ()
            {
                Audio.Play("event:/ui/main/button_back");
                level.Pause(returnIndex, minimal, false);
                menu.Close();
            });
            menu.OnPause = delegate ()
            {
                Audio.Play("event:/ui/main/button_back");
                level.Paused = false;
                Engine.FreezeTimer = 0.15f;
                menu.Close();
            };
            (level as Scene).Add(menu);

            void EnterSound()
            {
                //bool flag = MenuOptions.inGame && MenuOptions.snapshot != null;
                if (true)
                {
                    Audio.EndSnapshot(PauseSnapshot);
                }
            }

            void LeaveSound()
            {
                Audio.ResumeSnapshot(PauseSnapshot);
            }
        }
        
    }  
}   

