﻿using Celeste;
using Celeste.Mod;
using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using System.Collections;

namespace VanillaTweaks
{
    class VanillaTweaksHooks
    {

        public static void Load()
        {
            On.Celeste.OuiFileSelectSlot.Show += OuiFileSelectSlot_Show;
            On.Celeste.OuiFileSelectSlot.EnterFirstArea += OuiFileSelectSlot_EnterFirstArea;
            On.Celeste.Player.Update += Player_Update;
            On.Celeste.LevelEnter.Routine += LevelEnter_Routine;
            On.Celeste.Spikes.ctor_Vector2_int_Directions_string += Spikes_ctor_Vector2_int_Directions_string;
            On.Celeste.Spikes.OnEnable += Spikes_OnEnable;
            On.Celeste.GameplayStats.Render += GameplayStats_Render;
            On.Celeste.LookoutBlocker.ctor += LookoutBlocker_ctor;
        }

        private static void LookoutBlocker_ctor(On.Celeste.LookoutBlocker.orig_ctor orig, LookoutBlocker self, EntityData data, Vector2 offset)
        {
            orig(self, data, offset);
            if (VanillaTweaksModule.Settings.NoLookoutBlockers)
            {
                self.Collider = new Hitbox(0, 0);
            }
        }

        private static void GameplayStats_Render(On.Celeste.GameplayStats.orig_Render orig, GameplayStats self)
        {
            bool last = SaveData.Instance.CheatMode;
            if (VanillaTweaksModule.Settings.ForcePauseScreenStrawberryDisplay)
                SaveData.Instance.CheatMode = true;
            orig(self);
            SaveData.Instance.CheatMode = last;
        }

        private static void OuiFileSelectSlot_Show(On.Celeste.OuiFileSelectSlot.orig_Show orig, OuiFileSelectSlot self)
        {
            orig(self);
            if (VanillaTweaksModule.Instance.TotalStrawbs == -1)
            {
                VanillaTweaksModule.Instance.TotalStrawbs = VanillaTweaksModule.Instance.CountAllStrawbs() - 175;
            }
            if (VanillaTweaksModule.Settings.SaveFileShowTotalStrawberries)
            {
                DynData<StrawberriesCounter> dynData = new DynData<StrawberriesCounter>(self.Strawberries);

                int amount;
                if (self.SaveData != null)
                {
                    if (self.SaveData.LastArea_Safe.LevelSet == "Celeste")
                    {
                        amount = VanillaTweaksModule.Instance.CountVanillaCollectedStrawbs(self.SaveData);
                        self.Strawberries.OutOf = 175;
                        //dynData.Set("sOutOf", "/175");
                    }
                    else
                    {
                        amount = VanillaTweaksModule.Instance.CountCollectedStrawbs(self.SaveData) - VanillaTweaksModule.Instance.CountVanillaCollectedStrawbs(self.SaveData);
                        self.Strawberries.OutOf = VanillaTweaksModule.Instance.TotalStrawbs;
                        //dynData.Set("sOutOf", "/" + VanillaTweaksModule.Instance.TotalStrawbs.ToString());
                    }
                    self.Strawberries.Amount = amount;
                    //dynData.Set("sAmount", amount.ToString());
                    self.Strawberries.ShowOutOf = true;
                }
            }
        }

        public static Color[] BOX_COLORS = new Color[]
        {
            new Color(1f, 0.2f, 0.2f),
            new Color(0.2f, 1f, 0.2f),
            new Color(0.2f, 0.2f, 1f)
        };

        private static void Spikes_OnEnable(On.Celeste.Spikes.orig_OnEnable orig, Spikes self)
        {
            if (VanillaTweaksModule.Settings.ChangeRegularSpikesToOutline && self.DisabledColor == BOX_COLORS[0] * 0.5f || self.DisabledColor == BOX_COLORS[1] * 0.5f || self.DisabledColor == BOX_COLORS[2] * 0.5f)
            {
                self.EnabledColor = Color.White;
            }

            orig(self);
        }

        private static void Spikes_ctor_Vector2_int_Directions_string(On.Celeste.Spikes.orig_ctor_Vector2_int_Directions_string orig, Spikes self, Vector2 position, int size, Spikes.Directions direction, string type)
        {
            if (type == "default" && VanillaTweaksModule.Settings.ChangeRegularSpikesToOutline)
                type = "outline";
            orig(self, position, size, direction, type);
        }

        public static void Unload()
        {
            On.Celeste.OuiFileSelectSlot.Show -= OuiFileSelectSlot_Show;
            On.Celeste.OuiFileSelectSlot.EnterFirstArea -= OuiFileSelectSlot_EnterFirstArea;
            On.Celeste.Player.Update -= Player_Update;
            On.Celeste.LevelEnter.Routine -= LevelEnter_Routine;
            On.Celeste.Spikes.ctor_Vector2_int_Directions_string -= Spikes_ctor_Vector2_int_Directions_string;
            On.Celeste.Spikes.OnEnable -= Spikes_OnEnable;
            On.Celeste.GameplayStats.Render -= GameplayStats_Render;
            On.Celeste.LookoutBlocker.ctor -= LookoutBlocker_ctor;
        }

        

        private static void OuiFileSelectSlot_EnterFirstArea(On.Celeste.OuiFileSelectSlot.orig_EnterFirstArea orig, OuiFileSelectSlot self)
        {
            AreaData area = AreaData.Areas[SaveData.Instance.LastArea_Safe.ID];
            if (area.GetLevelSet() == "Celeste" && VanillaTweaksModule.Settings.SkipPrologueOnNewGame)
            {
                if (SaveData.Instance.LastArea_Safe.ID == 0)
                {
                    LevelSetStats levelSetStatsFor = SaveData.Instance.GetLevelSetStatsFor("Celeste");
                    levelSetStatsFor.UnlockedAreas = 1;
                    levelSetStatsFor.AreasIncludingCeleste[0].Modes[0].Completed = true;
                    SaveData.Instance.LastArea_Safe.ID = 1;
                }
            }
            orig(self);
        }

        static bool waitingForSkipPostcardUpdate;
        static bool debugSkipPostcard;
        static bool completeSkipPostcard;
        static int areaIDSkipPostcard;

        private static IEnumerator LevelEnter_Routine(On.Celeste.LevelEnter.orig_Routine orig, LevelEnter self)
        {
            DynData<LevelEnter> data = new DynData<LevelEnter>(self);
            Session session = data.Get<Session>("session"); //(Engine.Scene as Level).Session;

            debugSkipPostcard = SaveData.Instance.DebugMode;
            completeSkipPostcard = SaveData.Instance.Areas_Safe[session.Area.ID].Modes[0].Completed;
            areaIDSkipPostcard = session.Area.ID;
            if (VanillaTweaksModule.Settings.SkipPostcards)
            {
                SaveData.Instance.Areas_Safe[session.Area.ID].Modes[0].Completed = true;
                SaveData.Instance.DebugMode = false;
                waitingForSkipPostcardUpdate = true;
            }

            return orig(self);
        }

        private static float timeTillCameraDrop = 1.0f;
        private static float timeTillCameraRaise = 1.0f;
        private static bool cameraDropped = false;
        private static bool cameraRaised = false;
        private static bool snappedCam = false;

        private static void Player_Update(On.Celeste.Player.orig_Update orig, Player self)
        {
            if (waitingForSkipPostcardUpdate)
            {
                //Logger.Log(VanillaTweaksModule.ModName, $"deed it. Debug: {SaveData.Instance.DebugMode} {debugSkipPostcard}, Complete: {SaveData.Instance.Areas_Safe[areaIDSkipPostcard].Modes[0].Completed} {completeSkipPostcard}");
                SaveData.Instance.DebugMode = debugSkipPostcard;
                SaveData.Instance.Areas_Safe[areaIDSkipPostcard].Modes[0].Completed = completeSkipPostcard;
                //Logger.Log(VanillaTweaksModule.ModName, $"deed it agin. Debug: {SaveData.Instance.DebugMode} {debugSkipPostcard}, Complete: {SaveData.Instance.Areas_Safe[areaIDSkipPostcard].Modes[0].Completed} {completeSkipPostcard}");
                waitingForSkipPostcardUpdate = false;
            }
            // down
            if (Input.Aim.Value.Y > 0)
            {
                timeTillCameraRaise = 1.0f;
                if (self.OnGround() && !cameraDropped && VanillaTweaksModule.Settings.DuckToLookDown)
                {
                    timeTillCameraDrop -= Engine.DeltaTime;
                    if (timeTillCameraDrop < 0)
                    {
                        if (!snappedCam)
                            self.SceneAs<Level>().Camera.Y += VanillaTweaksModule.Settings.LookDownOffset / 8;
                        float camHeight = self.SceneAs<Level>().Camera.Bottom - self.SceneAs<Level>().Camera.Y;
                        if (self.SceneAs<Level>().Camera.Bottom > self.SceneAs<Level>().Bounds.Bottom || snappedCam)
                        {
                            self.SceneAs<Level>().Camera.Y = self.SceneAs<Level>().Bounds.Bottom - camHeight;

                            snappedCam = true;
                        }
                    }
                }
                else
                {
                    timeTillCameraDrop = 1.0f;
                    snappedCam = false;
                }
            }
            else
            {
                // Not holding down, reset values
                timeTillCameraDrop = 1.0f;
                snappedCam = false;
            }
            // Now handle looking upwards
            if (Input.Aim.Value.Y < 0)
            {
                if (self.OnGround() && !cameraRaised && VanillaTweaksModule.Settings.UpToLookUp)
                {
                    timeTillCameraRaise -= Engine.DeltaTime;
                    if (timeTillCameraRaise < 0)
                    {
                        //cameraRaised = true;
                        //self.SceneAs<Level>().CameraOffset.Y -= Settings.LookUpOffset;
                        self.SceneAs<Level>().Camera.Y -= VanillaTweaksModule.Settings.LookUpOffset / 8;
                        if (self.SceneAs<Level>().Camera.Y < self.SceneAs<Level>().Bounds.Top)
                        {
                            self.SceneAs<Level>().Camera.Y = self.SceneAs<Level>().Bounds.Top;
                        }
                    }
                }
            }
            else
            {
                // Not holding up, reset values
                timeTillCameraRaise = 1.0f;
            }

            orig(self);
        }
    }
}
