﻿using System;
using System.Collections.Generic;
using Celeste;
using Celeste.Mod;
using FMOD.Studio;
using Microsoft.Xna.Framework;
using Monocle;

namespace VanillaTweaks
{
    public class VanillaTweaksModuleSettings : EverestModuleSettings
    {
        [SettingIgnore]
        public bool Enabled { get; set; } = true;
        [SettingIgnore]
        public bool ShowJukebox { get; set; } = true;

        [SettingIgnore]
        public bool SaveFileShowTotalStrawberries { get; set; } = true;
        [SettingIgnore]
        public bool ForcePauseScreenStrawberryDisplay { get; set; } = true;
        //public bool ShowAllHeartsInJournal { get; set; } = true;
        [SettingIgnore]
        public bool DuckToLookDown { get; set; } = true;
        
        [SettingIgnore]
        public int LookDownOffset { get; set; } = 16;

        [SettingIgnore]
        public bool UpToLookUp { get; set; } = true;
        [SettingRange(0, 80)]
        [SettingIgnore]
        public int LookUpOffset { get; set; } = 16;

        [SettingIgnore]
        public bool SkipPostcards { get; set; } = true;
        [SettingIgnore]
        public bool SkipPrologueOnNewGame { get; set; } = true;
        [SettingIgnore]
        public bool ChangeRegularSpikesToOutline { get; set; } = true;

        [SettingIgnore]
        public bool NoLookoutBlockers { get; set; } = false;
    }

    public class VanillaTweaksModule : EverestModule
    {
        public const string ModName = "Vanilla Tweaks";
        //public static bool overwroteMusic;

        [Command("collected_berries", "[Vanilla Tweaks] Count all beries collected on this save file")]
        private static void CountCollectedBerries(bool showCompleted = true, bool showCampaign = true)
        {   
            int num = 0;
            int totalNum = 0;
            int i = 0;
            List<AreaStats> Areas_Safe = SaveData.Instance.Areas_Safe;
            Logger.Log(ModName, "Collected Strawberries:");
            foreach (AreaStats areaStat in Areas_Safe)
            {
                //MapData mapData = new MapData(new AreaKey(i, AreaMode.Normal));
                MapData mapData = AreaData.Get(i).Mode[0].MapData;
                num += areaStat.TotalStrawberries;
                totalNum += mapData.DetectedStrawberries;

                string text = "";
                if (showCampaign)
                {
                    string levelSet = areaStat.LevelSet;
                    text += $"{GetCampaignName(levelSet)}: ";
                }
                string areaName = AreaData.Get(mapData.Area).Name;

                text += $"{Dialog.Get(areaName)} => {areaStat.TotalStrawberries}/{mapData.DetectedStrawberries}";
                
                if (areaStat.TotalStrawberries == mapData.DetectedStrawberries)
                {
                    if (showCompleted)
                        Engine.Commands.Log(text, Color.Green);
                }
                else if (areaStat.TotalStrawberries > mapData.DetectedStrawberries)
                {
                    if (showCompleted)
                        Engine.Commands.Log(text, Color.Yellow);
                }
                else
                {
                    Engine.Commands.Log(text, Color.White);
                }
                Logger.Log(ModName, text);
                i++;
            }
            Engine.Commands.Log($"Total Collected Strawberries: {num}/{totalNum}", Color.Red);
        }

        [Command("collected_cassettes", "[Vanilla Tweaks] Count all cassettes collected on this save file")]
        private static void CountCollectedCassettes(bool showCampaign = true)
        {
            Color yellow = Color.Yellow;
            int num = 0;
            int i = 0;
            List<AreaStats> Areas_Safe = SaveData.Instance.Areas_Safe;
            Logger.Log(ModName, "Collected Cassettes:");
            foreach (AreaStats areaStat in Areas_Safe)
            {
                MapData mapData = new MapData(new AreaKey(i, AreaMode.Normal));

                num += areaStat.Cassette ? 1 : 0;
                if (areaStat.Cassette)
                {
                    string text = "";
                    string areaName = AreaData.Get(mapData.Area).Name;
                    if (showCampaign)
                    {
                        string levelSet = areaStat.LevelSet;
                        text += $"{GetCampaignName(levelSet)}: ";
                    }
                    text += $"{Dialog.Get(areaName)}";
                    Engine.Commands.Log(text, yellow);
                    Logger.Log(ModName, text);
                }
                i++;
            }
            Engine.Commands.Log("Total Collected Cassettes: " + num, Color.Red);
        }

        [Command("collected_hearts", "[Vanilla Tweaks] Count all hearts collected on this save file")]
        private static void CountCollectedHearts(bool showCampaign = true)
        {
            int num = 0;
            int totalNum = CountTotalHearts();
            List<AreaStats> Areas_Safe = SaveData.Instance.Areas_Safe;
            Logger.Log(ModName, "Collected Crystal Hearts:");
            foreach (AreaStats areaStats in Areas_Safe)
            {
                for (int i = 0; i < areaStats.Modes.Length; i++)
                {
                    if (areaStats.Modes[i] != null && areaStats.Modes[i].HeartGem)
                    {
                        string poemID = AreaData.Get(areaStats.SID).Mode[i].PoemID;
                        string poemText = "";
                        if (showCampaign)
                        {
                            string levelSet = areaStats.LevelSet;
                            poemText += $"{GetCampaignName(levelSet)}: ";
                        }
                        if (i == 2)
                        {
                            // C-Sides don't have names, so let's make our own
                            poemText += $"{Dialog.Clean(AreaData.Get(areaStats.SID).Name)} {Dialog.Clean("OVERWORLD_REMIX2")}";
                        } else
                        {
                            poemText += Dialog.Clean("poem_" + poemID);
                        }
                        
                        num++;
                        Color color;
                        switch (i)
                        {
                            case 0:
                                color = Color.LightBlue;
                                break;
                            case 1:
                                color = Color.Red;
                                break;
                            default:
                                color = Color.Yellow;
                                break;
                        }
                        Engine.Commands.Log(poemText, color);
                    }
                }
            }
            Engine.Commands.Log($"Total Collected Hearts: {num}/{totalNum}", Color.White);
        }

        [Command("jukebox_play", "[Vanilla Tweaks] Play a music track in the jukebox")]
        private static void JukeboxPlay(string song)
        {
            Jukebox.mutedSong = Audio.CurrentMusic;
            Audio.SetMusic(SFX.EventnameByHandle(song), true, true);
            Jukebox.playing = true;
            Jukebox.playingSong = SFX.EventnameByHandle(song);
        }

        [Command("jukebox_progress", "[Vanilla Tweaks] Set the music progress value for the track currently playing in the jukebox")]
        private static void CmdJukeboxProgress(int progress)
        {
            if (!Jukebox.playing)
            {
                float finalvalue;
                float value;
                Audio.CurrentMusicEventInstance.getParameterValue("progress", out value, out finalvalue);
                Jukebox.mutedSongProgress = (int)value;
            }
            Jukebox.playing = false;
            Audio.SetMusicParam("progress", progress);
            Jukebox.playing = true;
        }

        [Command("jukebox_stop", "[Vanilla Tweaks] Stop music currently played in the jukebox")]
        private static void JukeboxStop()
        {
            Jukebox.playing = false;
            //Audio.SetMusic(SFX.EventnameByHandle(Jukebox.prevSong), true, true);
            Audio.SetMusic(Jukebox.mutedSong, true, true);
            Audio.SetMusicParam("progress", Jukebox.mutedSongProgress);
        }

        private static int CountTotalHearts()
        {
            int num = 0;
            for (int i = 0; i < AreaData.Areas.Count; i++)
            {
                MapData mapData = AreaData.GetMode(AreaData.Areas[i].ToKey()).MapData;
                num += mapData.DetectedHeartGem ? 1 : 0;
            }

            return num;
        }

        private static string GetCampaignName(string levelSet)
        {
            if (levelSet.Contains("Celeste"))
            {
                return "Celeste";
            }
            else
            {
                return Dialog.Get(levelSet);
            }
        }


        // Only one alive module instance can exist at any given time.
        public static VanillaTweaksModule Instance;

        public VanillaTweaksModule()
        {
            Instance = this;
        }
        // no save data needed
        public override Type SaveDataType => null;

        public override Type SettingsType => typeof(VanillaTweaksModuleSettings);

        public static VanillaTweaksModuleSettings Settings => (VanillaTweaksModuleSettings)Instance._Settings;

        public override void LoadContent(bool firstLoad)
        {
            //FrostTempleModule.SpriteBank = new Monocle.SpriteBank(GFX.Game, "Graphics/FrostHelper/CustomSprites.xml");
        }

        public void QuickRetry()
        {
            Engine.TimeRate = 1f;
            Distort.GameRate = 1f;
            Distort.Anxiety = 0f;
            Level level = Engine.Scene as Level;
            Player player = Engine.Scene.Tracker.GetEntity<Player>();
            level.InCutscene = (level.SkippingCutscene = false);
            level.RetryPlayerCorpse = player.Die(Vector2.Zero, true, true);
            foreach (Component component in Engine.Scene.Tracker.GetComponents<LevelEndingHook>())
            {
                LevelEndingHook levelEndingHook = (LevelEndingHook)component;
                levelEndingHook.OnEnd?.Invoke();
            }
        }

        public int CountCollectedStrawbs(SaveData data)
        {
            int num = 0;
            
            List<AreaStats> Areas_Safe = data.Areas_Safe;
            foreach (AreaStats areaStat in Areas_Safe)
            {
                num += areaStat.TotalStrawberries;
            }
            return num;
        }

        public int TotalStrawbs = -1;
        public int CountAllStrawbs()
        {
            int num = 0;
            for (int i = 0; i < AreaData.Areas.Count; i++)
            {
                MapData mapData = AreaData.GetMode(AreaData.Areas[i].ToKey()).MapData;
                num += mapData.Strawberries.Count;
            }
            return num;
        }

        public int CountVanillaCollectedStrawbs(SaveData data)
        {
            int num = 0;
            List<AreaStats> Areas_Safe = data.Areas;
            foreach (AreaStats areaStat in Areas_Safe)
            {
                if (areaStat.SID.Contains("Celeste"))
                    num += areaStat.TotalStrawberries;
            }
            return num;
        }

        // Optional, initialize anything after Celeste has initialized itself properly.
        public override void Initialize()
        {
            Jukebox.GetByHandle();
        }


        bool Hooked = false;
        // Set up any hooks, event handlers and your mod in general here.
        // Load runs before Celeste itself has initialized properly.
        public override void Load()
        {
            if (!Hooked)
            {
                VanillaTweaksHooks.Load();
                Jukebox.Load();
                Hooked = true;
            }
            
        }

        public override void CreateModMenuSection(TextMenu menu, bool inGame, EventInstance snapshot)
        {
            base.CreateModMenuSection(menu, inGame, snapshot);
            menuItems = new List<TextMenu.Item>();
            menu.Add(new TextMenu.OnOff("Enabled", Settings.Enabled).Change(v => {
                Settings.Enabled = v;
                //refreshOptionMenuEnabledStatus();
                if (v)
                {
                    if (!Hooked)
                    {
                        VanillaTweaksHooks.Load();
                        Hooked = true;
                    }
                } else
                {
                    Hooked = false;
                    VanillaTweaksHooks.Unload();
                }
                RefreshOptionMenuEnabledStatus();
                // (de)activate all hooks!
                //if (v) ExtendedVariantsModule.Instance.HookStuff();
                //else ExtendedVariantsModule.Instance.UnhookStuff();

            }));
            menu.Add(new TextMenu.OnOff("Show Jukebox in Pause Screen", Settings.ShowJukebox).Change(v =>
            {
                Settings.ShowJukebox = v;
            }));
            CreatePauseMenuButtons(menu);
            RefreshOptionMenuEnabledStatus();

        }

        void CreatePauseMenuButtons(TextMenu menu)
        {
            TextMenu.Item item;
            menu.Add(item = new TextMenu.OnOff("Total Strawberries in Save File Select", Settings.SaveFileShowTotalStrawberries).Change(v =>
            {
                Settings.SaveFileShowTotalStrawberries = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Force Pause Screen Strawberry Display", Settings.ForcePauseScreenStrawberryDisplay).Change(v =>
            {
                Settings.ForcePauseScreenStrawberryDisplay = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Duck to look down", Settings.DuckToLookDown).Change(v =>
            {
                Settings.DuckToLookDown = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.Slider("Look down offset", delegate (int i)
            {
                return Settings.LookDownOffset.ToString();
            }, 0, 100, Settings.LookDownOffset).Change(v =>
            {
                Settings.LookDownOffset = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Hold up to look up", Settings.UpToLookUp).Change(v =>
            {
                Settings.UpToLookUp = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.Slider("Look up offset", delegate (int i)
            {
                return Settings.LookUpOffset.ToString();
            }, 0, 100, Settings.LookUpOffset).Change(v =>
            {
                Settings.LookUpOffset = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Skip Postcards", Settings.SkipPostcards).Change(v =>
            {
                Settings.SkipPostcards = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Skip Prologue on new game", Settings.SkipPrologueOnNewGame).Change(v =>
            {
                Settings.SkipPrologueOnNewGame = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("Regular spikes to outline", Settings.ChangeRegularSpikesToOutline).Change(v =>
            {
                Settings.ChangeRegularSpikesToOutline = v;
            }));
            menuItems.Add(item);
            menu.Add(item = new TextMenu.OnOff("No Lookout Blockers", Settings.NoLookoutBlockers).Change(v =>
            {
                Settings.NoLookoutBlockers = v;
            }));
            menuItems.Add(item);
        }

        private List<TextMenu.Item> menuItems = new List<TextMenu.Item>();
        private void RefreshOptionMenuEnabledStatus()
        {
            foreach (TextMenu.Item item in menuItems)
            {
                if (item != null) item.Visible = Settings.Enabled;
            }
        }

        private void OuiJournalPoem_ctor(On.Celeste.OuiJournalPoem.orig_ctor orig, OuiJournalPoem self, OuiJournal journal)
        {

            List<string> prevPoemSafe = SaveData.Instance.Poem_Safe;
            SaveData.Instance.Poem_Safe = new List<string>();
            foreach (AreaStats areaStats in SaveData.Instance.Areas_Safe)
            {
                // -1 because we don't want C-Sides
                for (int i = 0; i < areaStats.Modes.Length-1; i++)
                {
                    if (areaStats.Modes[i] != null && areaStats.Modes[i].HeartGem)
                    {
                        string poemID = AreaData.Get(areaStats.SID).Mode[i].PoemID;
                        string poemText = "";
                        poemText += Dialog.Clean("poem_" + poemID);
                        SaveData.Instance.Poem_Safe.Add(poemText);
                    }
                }
            }
            orig(self, journal);
            SaveData.Instance.Poem_Safe = prevPoemSafe;
        }
        
        // Unload the entirety of your mod's content, remove any event listeners and undo all hooks.
        public override void Unload()
        {
            Hooked = false;
            VanillaTweaksHooks.Unload();
            Jukebox.Unload();
        }
    }
}
